#!/usr/bin/env Rscript

#args <- commandArgs(trailingOnly=TRUE)

# need parameters - user_params (list):
# int_files (intensity file names), int_formats, int_platforms,
# norm_mtd: quantile normalization.  

library(affy)
library(limma)

pfs_allowed <- c('U95Av2', 'U133A', 'U133P2')
fmts_allowed <- c('cel', 'user_defined')

readData <- function(fns, fmts, fmts_allowed=fmts_allowed, path=NULL) {
	# fns and fmts are vectors
	fmts <- tolower(fmts)
	if (length(setdiff(fmts, fmts_allowed))>0) stop(sprintf('Error: Unsupported intensity file formats ("%s") involved!', paste(setdiff(fmts, fmts_allowed), collapse='", "')))
	fmts <- rep(fmts, length=length(fns))
	DT <- list()
	for (i in seq(fns)) {
		fn <- if(is.null(path)) fns[i] else file.path(path, fns[i])
		if (fmts[i] == 'user_defined') {
			DTMP <- read.delim(fn, as.is=TRUE)
			cnm <- tolower(colnames(DTMP))
			colnames(DTMP) <- cnm
			if (any(!(c('id', 'ch1.intensity') %in% cnm))) stop(paste('Error: Necessary column ("ID" or "ch1.Intensity") not found in the file ', fn, '!', sep=''))
			rnm <- DTMP$id
			DTMP <-  cbind(DTMP$ch1.intensity)
			rownames(DTMP) <- rnm
			}
		else {
			DTMP <- exprs(rma(ReadAffy(filenames=fn)))
			}
		DT[[i]] <- DTMP
		}
	invisible(DT)
	}
mkDataM <- function(DTL) {
	# DTL is a list of vector, matrix or data.frame, can be output from readData
	# require all data come from the same platform
	if (length(DTL)==0) return(NULL)
	nncol <- function(x) if (length(dim(x))<2) 1 else dim(x)[2]
	nnrow <- function(x) if (length(dim(x))<2) length(x) else dim(x)[1]
	rnames <- function(x) if (is.null(dim(x))) names(x) else rownames(x)
	lnms <- names(DTL)
	cnames <- function(i) if (is.null(dim(DTL[[i]]))) lnms[i] else colnames(DTL[[i]])
	cnm <- unlist(lapply(seq(DTL), cnames), recursive=TRUE)
	DTMP <- DTL[[1]]
	DT <- matrix(nrow=nnrow(DTMP), ncol=sum(sapply(DTL, nncol)))
	cur_col <- nncol(DTMP)
	DT[, seq(cur_col)] <- DTMP
	rnm1 <- rnames(DTMP)
	rownames(DT) <- rnm1
	colnames(DT) <- cnm
	for (DTMP in DTL[-1]) {
		rnm <- rnames(DTMP)
		if (!setequal(rnm, rnm1)) stop('Error: Different row names across arrays!')
		n_col <- nncol(DTMP)
		cur_col <- cur_col + seq(n_col)
		DT[, cur_col] <- DTMP[match(rnm1, rnm), ]
		cur_col <- cur_col[length(cur_col)]
		}
	invisible(DT)
	}

getQuantileRef <- function(M, fn='quantile_ref.txt') {
	M <- apply(M, 2, sort)
	quantile_ref <- cbind(rowMeans(M))
	#rownames(quantile_ref) <- rownames(M)
	#save(quantile_ref, file=fn)
	write.table(quantile_ref, file=fn, quote=FALSE, row.names=FALSE)
	invisible(quantile_ref)
	}

normQuanile <- function(M, quantile_ref=NULL) {
	if (is.null(quantile_ref)) quantile_ref <- read.delim('quantile_ref.txt')[[1]] #load('quantile_ref.Rdata')
	dnms <- dimnames(M)
	M <- apply(M, 2, function(x) quantile_ref[rank(x)])
	dimnames(M) <- dnms
	invisible(M)
	}

getID95_133 <- function(Fn133A, Fn133P2, fn='Affy_map_U133_U95.txt') {
	U133A <- rownames(exprs(rma(ReadAffy(filenames=Fn133A)))) # have to use filenames for for variable, 
	U133P2 <- rownames(exprs(rma(ReadAffy(filenames=Fn133P2))))
	U133 <- intersect(U133A, U133P2)
	save(U133, file='ID133.Rdata')

	DT <- read.delim(fn, stringsAsFactors=FALSE) # or colClasses = "character". But as.is=TRUE) will lead to erro: unexpected symbol
	flt <- which(DT$A.Array.Name == 'HG-U95Av2')
	DT <- DT[flt,]
	ID_outP2 <- setdiff(DT$U133ProbeSetName, ID133P2)
	if (any(ID_outP2)) { # remove probes not in U133_Plus 2.0
		DTMP <- DT[-match(ID_outP2, DT$U133ProbeSetName), ]
		IDS <- list(U95Av2=DTMP$U95ProbeSetName, U133P2=DTMP$U133ProbeSetName)
		}
	else 
		IDS <- list(U95Av2=DT$U95ProbeSetName, U133P2=DT$U133ProbeSetName)
	save(IDS, file='ID95_133P2.Rdata')

	flt <- which(DT$B.Array.Name == 'HG-U133A')
	DT <- DT[flt,]
	IDS <- list(U95Av2=DT$U95ProbeSetName, U133A=DT$U133ProbeSetName)
	save(IDS, file='ID95_133A.Rdata')

	flt <- which(DT$U133ProbeSetName %in% ID133)
	DT <- DT[flt,]
	IDS <- list(U95Av2=DT$U95ProbeSetName, U133=DT$U133ProbeSetName)
	save(IDS, file='ID95_133.Rdata')
	
	invisible(list(ID133=ID133, ID95_133=ID95_133, ID95_133A=ID95_133A, ID95_133P2=ID95_133P2))
	}

matchData <- function(DTL, PFL, pfs_allowed=pfs_allowed) {
	# match data from different platform
	# DTL is a list of vector, matrix or data.frame, can be output from readData
	# PFL is a vector/list of platform names of DTL
	PFL <- unlist(PFL)
	if (length(DTL) != length(PFL)) stop('Error: platform names doesn\'t match array number!')
	upfs <- unique(unlist(PFL)) 
	if (any(setdiff(upfs, pfs_allowed))) stop('Error: Unsupported platforms involved!')
	if (length(upfs) <= 1) { # all are the same platform
		if (length(DTL) == 0) return(DTL)
		if (length(DTL) == 1) return(DTL[[1]])
		DT <- DTL[[1]]
		rnms <- rownames(DT)
		for (i in 2:length(DTL)) {
			#DT <- cbind(DT, DTL[[i]])
			DT <- cbind(DT, DTL[[i]][match(rownames(DTL[[i]]), rnms), ])
			}
		return(DT)
		}
	if (setequal(upfs, c('U133A', 'U133P2'))) {
		load('ID133.Rdata') # ID133: a vector of ID commonly shared by all U133 arrays
		DT <- NULL
		for (i in seq(DTL)) {
			DTMP <- DTL[[i]]
			if (is.null(dim(DTMP))) DT <- cbind(DT, DTMP[match(U133, names(DTMP))])
			else if (length(dim(DTMP))<=2) DT <- cbind(DT, DTMP[match(U133, colnames(DTMP))])
			else stop('Error: Too many dimenstions in the data!')
			} 
		rownames(DT) <- ID133
		return(DT)
		}
	# must contain U95Av2, U133XX
	if (setequal(upfs, c('U95Av2', 'U133A'))) load('ID95_133A.Rdata')
	else if (setequal(upfs, c('U95Av2', 'U133P2'))) load('ID95_133P2.Rdata')
	else if (setequal(upfs, c('U95Av2', 'U133A', 'U133P2'))) {
		load('ID95_133.Rdata')
		IDS$U133A <- IDS$U133P2 <- IDS$U133
		}
	DT <- NULL
	for (i in seq(DTL)) {
		DTMP <- DTL[[i]]
		PF <- PFL[i]
		if (is.null(dim(DTMP))) DT <- cbind(DT, DTMP[match(IDS[[PF]], names(DTMP))])
		else if (length(dim(DTMP))<=2) DT <- cbind(DT, DTMP[match(IDS[[PF]], colnames(DTMP))])
		else stop('Error: Too many dimenstions in the data!')
		}
	invisible(DT)
	}

has_snow <- require(snow)
nodes <- if (has_snow) switch(getClusterOption("type"), PVM=8, MPI=8, SOCK=paste('dell', c(5,4,5,4,5,4,5,4), sep='')) else NULL
nodes <- 1
N_para <- 10000
a <- try(source('parTools.R'))
if ('try-error' %in% class(a)) computeM <- function(exprs, fun, min_proc=1) t(apply(exprs, 1, fun))
rm(a)
use_par <- TRUE

computeIt <- function(exprs, fun, min_proc=NULL) {
	if (is.null(min_proc)) min_proc <- nodes
	if (use_par && has_snow && nrow(exprs) > N_para) M_fit <- computeM(exprs, fun, min_proc=nodes) 
	else M_fit <- t(apply(exprs, 1, fun))
	M_fit
	}

trainModel <- function(DT, samps, facs=NULL, fac_sep='__LEVEL__') {
	# DT: gene as row and sample as column
	# samps: sample as row and tissue (T,S,B,A) as column
	# factors: should be a data.frame similar to samps
	n_gene <- nrow(DT)
	#T <- samps[, 'T']
	#S <- samps[, 'S']
	#M_fit <- computeIt(DT, function(x) lm(x ~ 1 + T + S)$coef[1:3])
	#colnames(M_fit) <- c('intercept', 'T', 'S')
	idx_fn <- match('filename', tolower(colnames(samps)))
	cnms_orig <- colnames(samps)[-idx_fn] #setdiff(colnames(samps), 'FileName')
	idx_v <- -idx_fn #match(cnms_orig, colnames(samps))
	cnms_v <- paste('V', seq(cnms_orig), sep='')
	colnames(samps)[idx_v] <- cnms_v
	attach(samps)
	fnmcoef <- NULL
	if (!is.null(facs)) {
		#idx_fn <- match('filename', tolower(colnames(facs)))
		facnms_orig <- colnames(facs)#[-idx_fn] 
		cnms_f <- paste('F', seq(facnms_orig), '_', sep='')
		#colnames(facs)[-idx_fn] <- cnms_f
		colnames(facs) <- cnms_f
		#for (cl in cnms_f) facs[, cl] <- as.factor(gsub('"', '.', facs[, cl]))
		for (i in seq(cnms_f)) {
			afac <- as.factor(gsub('[^a-zA-Z0-9_-]', '.', facs[, i])) # levels of the factors will be used as column names in the result file, so it is necessary to normalize the name in order that they can keep unchanged as data.frame colnames after read.delim
			lvi <- levels(afac)
			if(length(lvi)>1) {
				facs[, i] <- afac 
				fnmcoef <- c(fnmcoef, paste(facnms_orig[i], lvi[-1], sep=fac_sep))
				}
			else facs[, i] <- NULL # discard factors of only one level
			}
		if (ncol(facs) > 0) {
			cnms_f <- colnames(facs)
			attach(facs)
			}
		else facs <- NULL
		}

	mdl <- if(is.null(facs)) paste('lm(x ~ 1 + ', paste(cnms_v, collapse=' + '), ')', sep='') else paste('lm(x ~ 1 + ', paste(cnms_v, collapse=' + '), ' + ', paste(cnms_f, collapse=' + '), ')', sep='')
	M_fit <- computeIt(DT, function(x) eval(parse(text=mdl))$coef)
	colnames(M_fit) <- c('intercept', cnms_orig, fnmcoef) 
	rownames(M_fit) <- rownames(DT)
	invisible(M_fit)
	}

trainMain <- function(samp_file, fmts, model_file, factor_file=NULL, gene_num=100, path=NULL, fit_tail='.FIT', ref_tail='.REF') {
	# samp_file: a table file with sample as row, filename, tissue type (T,S,B,A) as column
	ref_file <- getRefFn(model_file, fit_tail=fit_tail, ref_tail=ref_tail)
	samps <- read.delim(samp_file)
	cols <- tolower(colnames(samps)) # "FileName", "T", "S"

	cols_need <- c('FileName')#, 'T', 'S')
	#cols_not_in <- !(toupper(cols_need) %in% cols)
	cols_not_in <- !(tolower(cols_need) %in% cols)
	if (any(cols_not_in)) stop(sprintf('Error: no column "%s" found in the sample file!', paste(cols_need[cols_not_in], collapse='", "')))
	if (ncol(samps) < 3) stop('Too few columns in the sample file!, at least two other columns are required except the column "FileName".')
	#if (!('FILENAME' %in%  colnames(samps))) stop('Error: no column "FileName" found in the sample file!')
	#fns <- if (is.null(path)) samps[, 'FILENAME'] else file.path(path, samps[, 'FILENAME'])
	fns <- samps[, match('filename', cols)] #'FILENAME']
	#basefns <- basename(fns)
	fn_dif <- setdiff(fns, dir(if(is.null(path)) '.' else path))
	if (length(fn_dif) > 0) stop(sprintf('Error: file "%s" not found!', paste(fn_dif, collapse='", "')))

	# find other factors
	facs <- NULL
	if (!is.null(factor_file)) {
		facs <- read.delim(factor_file, as.is=TRUE)
		cols <- tolower(colnames(facs))
		cols_not_in <- !(tolower(cols_need) %in% cols)
		if (any(cols_not_in)) stop(sprintf('Error: no column "%s" found in the factor file!', paste(cols_need[cols_not_in], collapse='", "')))
		if (ncol(samps) < 2) {
			warning('Warning: no factors defined in the factor file!')
			facs <- NULL }
		else {
			idx_fn <- match('filename', cols)
			#fns <- basename(fns)
			if (length(setdiff(fns, facs[, idx_fn])) > 0) stop('Error: cannot find the values of factors for some intensity files!')
			facs <- facs[match(fns, facs[, idx_fn]), ] # sort and screen factor values according to intensity file names.
			facs[, idx_fn] <- NULL
			}
		}
	

	#samps <- samps[match(basefns, fns), ] # only use those in int_files (and in the same order)
	# read intensity files
	DT <- readData(fns, fmts, fmts_allowed=fmts_allowed, path=path)  # a list
	DT <- mkDataM(DT) # a matrix
	DT <- normalizeBetweenArrays(DT, method='quantile')
	quantile_ref <- sort(DT[,1])
	#save(quantile_ref, file=ref_file)
	write.table(quantile_ref, file=ref_file, sep='\t', quote=FALSE, row.names=FALSE)
	DT <- fltDT(DT, samps, n_data=gene_num, perm=10)
	coef <- trainModel(DT, samps, facs)
	#save(coef, file=model_file)
	write.table(coef, file=model_file, sep='\t', quote=FALSE)
	}

fltDT <- function(DT, samps, n_data=NULL, v_cor=0.5, fun_flt=any, perm=1) {
	# select genes by correlation coefficient, the number of genes to use is n_data or determined by v_cor
	# DT is a matrix-like data set: rows for genes and columns for samples
	# samps is a matrix-like data set: rows for samples and columns for tissue types
	samps <- samps[, match(setdiff(tolower(colnames(samps)), 'filename'), tolower(colnames(samps)))]
	#ns <- ncol(samps)
	ns <- nrow(samps)
	ns1 <- as.integer(ns*0.9)
	#if (perm == 1 || ns <= 1) {
	if (perm == 1 || ns1 < 2 || ns <= 3) {
		v_cors <- abs(cor(t(DT), samps))
		#v_cors <- abs(cor(t(DT), samps[,setdiff(colnames(samps), 'FileName')]))
		#v_cors <- abs(cor(t(DT), samps[,c('T','S')]))
		if (is.null(n_data) || n_data <=0 ) flt <- which(apply(v_cors>=v_cor, 1, fun_flt))
		else {
			#flt <- rank(apply(v_cors, 1, sum)) <= n_data # rank will lead to uncertain number
			n_data <- min(n_data, nrow(v_cors))
			flt <- order(rowMeans(v_cors), decreasing=TRUE)[1:n_data]
			}
		}
	else {
		flt <- NULL
		for (i in seq(perm)) {
			idx <- sample(ns, ns1)
			v_cors <- abs(cor(t(DT[,idx]), samps[idx,]))
			if (is.null(n_data) || n_data <=0 ) flt <- c(flt, which(apply(v_cors>=v_cor, 1, fun_flt)))
			else {
				n_data <- min(n_data, nrow(v_cors))
				flt <- c(flt, order(rowMeans(v_cors), decreasing=TRUE)[1:n_data])
				}
			}
		if (is.null(n_data)) flt <- unique(flt)
		else {
			flt <- tapply(flt, flt, length)
			flt <- flt[order(flt, decreasing=TRUE)[1:n_data]]
			flt <- as.integer(names(flt))
			}
		}
	DT <- if(length(flt)>1) cbind(DT[flt,]) else rbind(DT[flt,])
	invisible(DT)
	}

normPerc <- function(x) {
	# to normalize x[1:4] into [0, 100] and the sum should be 100
	n <- length(x)
	#x <- x[1:2]
	x[x<0] <- 0
	x[x>100] <- 100
	x <- x*(100/sum(x))
	x <- round(x)
	x[n] <- 100-sum(x[-n])
	x
	}

predictPerc <- function(DT, coef, facs=NULL, fac_sep='__LEVEL__') {
	# DT: a data matrix: cols are samples (intensity file name), rows are genes
	# coef: a coefficient matrix of a linear model, rows are genes, columns are "intercept", cell types, and other factors.
	# facs: a data.frame or matrix, rows are samples with order same to those in DT's columns, columns are factor names, body are factor levels.
	intercept <- coef[,'intercept']
	nr <- length(intercept)
	#nc <- ncol(coef)
	#dm <- matrix(NA, nrow=ncol(DT), ncol=nc) # 3)
	#dimnm <- list(colnames(DT), paste(c('T', 'S', 'rest'), 'fit', sep='_') )
	cols <- colnames(coef)
	cnms_orig <- setdiff(cols, 'intercept')
	if (!is.null(facs)) {
		facnms <- colnames(facs)
		facs <- as.matrix(facs)
		for (i in seq(nrow(facs))) facs[i, ] <- paste(facnms, gsub('[^a-zA-Z0-9_-]', '.', facs[i, ]), sep = fac_sep)
		#cnms_orig <- setdiff(cols, c('intercept', facnms))
		cnms_orig <- cnms_orig[-grep(fac_sep, cnms_orig)]
		#facs <- t(facs)
		}
	nc <- length(cnms_orig) + 1 # 1 for intercept - rest
	dm <- matrix(NA, nrow=ncol(DT), ncol=nc) # 3)
	idx_v <- match(cnms_orig, colnames(coef))
	cnms_v <- paste('V', seq(cnms_orig), sep='')
	colnames(coef)[idx_v] <- cnms_v
	coef <- as.data.frame(coef)
	attach(coef)
	mdl <- paste('lm(x ~ 1 + ', paste(cnms_v, collapse=' + '), ')', sep='')
	dimnm <- list(colnames(DT), c(cnms_orig, 'rest') )
	#T <- coef[,'T']; S <- coef[,'S']
	for (i in 1:ncol(DT)) {
		x <- DT[,i] - intercept
		if (!is.null(facs)) {
			shared_cols <- intersect(colnames(coef),  facs[i,])
			if (length(shared_cols) > 0) 
				x <- x - rowSums(cbind(coef[, shared_cols]), na.rm=TRUE)
			}
		#ft <- lm(x ~ 1 + T + S) # -1 + T + S
		#dm[i,1:2] <- ft$coef[2:3] # 1:2
		dm[i, -nc] <- eval(parse(text=mdl))$coef[-1]
		}
	dm[,nc] <- 100 - rowSums(rbind(dm[,-nc])) #	intercept
	dm <- t(apply(dm, 1, normPerc ))
	dimnames(dm) <- dimnm
	invisible(dm)
	#invisible(list(exprs=DT, samps=cbind(samps, dm, distance=rowMeans(abs(samps[,c('T','S','B', 'A')]-dm)))))
	}

getRefFn <- function(mdl_fn, model_head='MODEL_', fit_tail='.FIT', ref_tail='.REF') {
	# mdl_fn is a file name
	# change .../MODEL_XXX.AAA to .../QR_XXX.BBB
	return(paste(substr(mdl_fn, 1, nchar(mdl_fn)-nchar(fit_tail)), ref_tail, sep=''))
	# change .../MODEL_XXX.XXX to .../QR_XXX.XXX
	pths <- strsplit(mdl_fn, .Platform$file.sep)[[1]]
	bnm <- pths[length(pths)] # or basename(mdl_fn)
	bnm <- substr(bnm, 4, nchar(bnm)) # remove "MOD"
	substr(bnm, 1, 2) <- 'QR'
	pths[length(pths)] <- bnm
	paste(pths, collapse=.Platform$file.sep)
	}

predictMain <- function(fns, fmts, model_file, factor_file=NULL, path=NULL, fit_tail='.FIT', ref_tail='.REF', fac_sep='__LEVEL__') {
	#if (!is.null(path)) fns <- file.path(path, fns) #sep=.Platform$file.sep
	DT <- readData(fns, fmts, fmts_allowed=fmts_allowed, path=path)  # a list
	DT <- mkDataM(DT) # a matrix
	cnm <- colnames(DT)
	refn <- getRefFn(model_file, fit_tail=fit_tail, ref_tail=ref_tail)
	# use XXX.Rdata
	#refv <- load(refn); DT <- normQuanile(DT, quantile_ref=get(refv)); rm(refv) # a matrix
	DT <- normQuanile(DT, quantile_ref=read.delim(refn)[[1]]) # a matrix
	# load fitted model
	fit <- as.matrix(read.table(model_file)) #load(model_file) # get fit
	gnms <- rownames(fit)
	fnms <- colnames(fit)
	idx <- grep(fac_sep, fnms)
	facs <- NULL
	if (length(idx) > 0) { # need factor file
		# find factors first
		facnms <- sapply(strsplit(fnms[idx], fac_sep), function(x) x[1])
		facnms <- tolower(facnms)
		if (is.null(factor_file)) stop('Error: the model used needs a factor file!')
		facs <- read.delim(factor_file, as.is=TRUE)
		cols <- tolower(colnames(facs))
		cols_need <- c('FileName')#, 'T', 'S')
		cols_not_in <- !(tolower(cols_need) %in% cols)
		if (any(cols_not_in)) stop(sprintf('Error: no column "%s" found in the factor file!', paste(cols_need[cols_not_in], collapse='", "')))
		difnms <- setdiff(c('filename', facnms), cols)
		if (length(difnms) > 0) stop(paste('Error: some necessary factors are not presented in the factor file:', paste(difnms, collapse=', ')))
		difnms <- setdiff(cols, c('filename', facnms))
		if (length(difnms) > 0) {
			warning(paste('Warning: some factors presented in the factor file will not be used:', paste(difnms, collapse=', ')))
			# remove extra factors
			facs <- facs[, -match(difnms, cols)]
			}
		idx_fn <- match('filename', cols)
		fns <- basename(fns)
		if (length(setdiff(fns, facs[, idx_fn])) > 0) stop('Error: cannot find the values of factors for some intensity files!')
		facs <- facs[match(fns, facs[, idx_fn]), ] # sort and screen factor values according to intensity file names.
		facs[, idx_fn] <- NULL # remove the FileName column
		}
	else if (!is.null(factor_file)) warning(paste('Warning: factor file', basename(factor_file), 'is not needed!'))
	DT <- if (length(gnms)==1) rbind(DT[match(gnms, rownames(DT)), ]) else cbind(DT[match(gnms, rownames(DT)), ]) # only use probes appeared in training data sets
	colnames(DT) <- cnm
	rlt <- predictPerc(DT, coef=fit, facs=facs, fac_sep=fac_sep)
	}


tisMain <- function(user_params) {
	# expected variables in user_params:
	# common:
	# 	analyze_type, intensity_dir, intensity_format
	# 	model_file_full (#model_dir, model_dir_sys, model_file)
	# 	fit_tail, ref_tail
	# for train:
	# 	perc_file, probe_num
	# for prediction:
	# 	intensity_file (one or more files)
	# 	result_dir, chart_dir, req_name

	#save(user_params, file='user_params.Rdata')
	attach(user_params)
	if (identical(analyze_type, 'train'))
		trainMain(samp_file=perc_file, fmts=intensity_format, model_file=model_file_full, factor_file=user_params$factor_file, gene_num=probe_num, path=intensity_dir, fit_tail=fit_tail, ref_tail=ref_tail)
	else if (identical(analyze_type, 'predict')) {
		rlt <- predictMain(intensity_file, intensity_format, model_file=model_file_full, factor_file=user_params$factor_file, path=intensity_dir, fit_tail=fit_tail, ref_tail=ref_tail)
		write.table(cbind(Experiment=rownames(rlt), rlt), file=file.path(result_dir, 'cell_percentages.txt'), sep='\t', quote=FALSE, row.names=FALSE)
		}
	NULL
	}

#load('user_params.Rdata')
#source('tispred.R')
#debug(trainModel)
#debug(predictMain)
#debug(predictPerc)

#tisMain(user_params)

